#include <QtCore/QAbstractFileEngineHandler>
#include <QtCore/QAbstractFileEngine>
#include <QtCore/QDateTime>
#include <QtCore/QDebug>

class TestFSIterator : public QAbstractFileEngineIterator {
private:
    int index;
    QStringList list;
public:
    TestFSIterator(const QString& parent, QDir::Filters filters,
                   const QStringList & filterNames)
        :QAbstractFileEngineIterator(filters, filterNames),
         index(-1) {
        list << ".." << ".";
        if (parent == "/bup") {
            list << "A" << "B" << "C";
        }
    }
    QString next() {
        index++;
        return currentFileName();
    }
    bool hasNext() const {
        return index + 1 < list.size();
    }
    QString currentFileName() const {
        return (index < list.size()) ?list[index] :QString();
    }
};

class TestFS : public QAbstractFileEngine {
private:
    QString path;
public:
    TestFS(const QString& path) :QAbstractFileEngine() {
        this->path = path;
    }
    ~TestFS() {
    }
    Iterator* beginEntryList (QDir::Filters filters, const QStringList & filterNames ) {
        qDebug() << "beginEntryList";
        return new TestFSIterator(path, filters, filterNames);
    }
    virtual FileFlags fileFlags (FileFlags type = FileInfoAll ) const {
        FileFlags flags = ExistsFlag | ReadOwnerPerm | ReadUserPerm |
                ReadGroupPerm | ReadOtherPerm | DirectoryType;
        return flags;
    }
    virtual QString fileName (FileName file = DefaultName ) const {
        QString name;
        switch (file) {
        case DefaultName:
            name = path;
            break;
        case BaseName:
            name = path.section('/', -1);
            break;
        }
        return name;
    }
    virtual void setFileName(const QString & file) {
        path = file;
    }
};
class TestFSHandler : public QAbstractFileEngineHandler {
public:
    TestFSHandler() :QAbstractFileEngineHandler() {
    }
    QAbstractFileEngine* create(const QString& path) const {
        return path.startsWith("/bup") ?new TestFS(path) :NULL;
    }
};
int
main() {
    TestFSHandler testfshandler;
    QDir d("/bup");
    foreach(const QString& name, d.entryList()) {
        qDebug() << name;
    }
    qDebug() << "entryList: " + QString::number(d.entryList().size());
    return 0;
}
